/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.command;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.fortiss.tooling.base.ui.dnd.DragContext;
import org.fortiss.tooling.base.ui.editpart.policy.ConnectionGraphicalNodeEditPolicy;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;

/**
 * This command is used by {@link ConnectionGraphicalNodeEditPolicy} to
 * communicate with the connector service.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: B0F33CCD646C725CE690BDB141193612
 */
public class ConnectionCommand extends Command {

	/** Stores DropContext information. */
	private final DragContext context;

	/** Reused connection (if the command is used for reconnection). */
	private final EObject connection;

	/** Constructor. */
	public ConnectionCommand(DragContext context, EObject connection) {
		this.context = context;
		this.connection = connection;
	}

	/** Sets the target edit part and location. */
	public void setTarget(EditPart targetObject, Point targetLocation) {
		context.setTarget(targetObject, targetLocation);
	}

	/** {@inheritDoc} */
	@Override
	public boolean canExecute() {
		if (context == null || context.getSource() == null
				|| context.getTarget() == null) {
			return false;
		}
		if (context.getSource().getModel() instanceof EObject
				&& context.getTarget().getModel() instanceof EObject) {
			return IConnectionCompositorService.INSTANCE.canConnect((EObject) context
					.getSource().getModel(), (EObject) context.getTarget()
					.getModel(), connection, context);
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public void execute() {
		IConnectionCompositorService.INSTANCE.connect((EObject) context.getSource()
				.getModel(), (EObject) context.getTarget().getModel(),
				connection, context);
	}
}
