/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.SnapToHelper;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;
import org.eclipse.gef.editpolicies.SnapFeedbackPolicy;
import org.eclipse.swt.SWT;
import org.fortiss.tooling.base.ui.editpart.policy.ConnectionGraphicalNodeEditPolicy;
import org.fortiss.tooling.base.ui.editpart.policy.DiagramLayoutEditPolicy;
import org.fortiss.tooling.base.ui.layout.util.SnapToGridUtils;

/**
 * The base class for graphical diagrams.
 * 
 * @param <T>
 *            the type of the model element shown.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 76BF89DC0EFA9511DDF95CA4EC32C074
 */
public abstract class DiagramEditPartBase<T extends EObject> extends
		GraphicalEditPartBase<T> {

	/** Stores the connection router. */
	private final ConnectionRouter connectionRouter;

	/** Constructor. */
	protected DiagramEditPartBase(T modelElement,
			ConnectionRouter connectionRouter) {
		super(modelElement);
		this.connectionRouter = connectionRouter;
	}

	/** Constructor. */
	protected DiagramEditPartBase(T modelElement) {
		this(modelElement, new BendpointConnectionRouter());
	}

	/** {@inheritDoc} */
	@Override
	protected IFigure createFigure() {
		setConnectionRouter();

		FreeformLayer f = new FreeformLayer();
		f.setOpaque(false);
		f.setLayoutManager(new XYLayout());
		f.setBackgroundColor(ColorConstants.white);

		return f;
	}

	/** {@inheritDoc} */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new RootComponentEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new DiagramLayoutEditPolicy());
		// snap to invisible grid policy
		if (enableSnapToGrid()) {
			installEditPolicy("SnapToGridFeedback", new SnapFeedbackPolicy());
		}
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new ConnectionGraphicalNodeEditPolicy());
	}

	/**
	 * Returns whether snap-to-grid should be enabled. The default is
	 * <code>true</code>.
	 */
	protected boolean enableSnapToGrid() {
		return true;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("rawtypes")
	@Override
	public Object getAdapter(Class adapter) {
		if (adapter == SnapToHelper.class && enableSnapToGrid()) {
			return SnapToGridUtils.snapToGridAdapter(this);
		}
		return super.getAdapter(adapter);
	}

	/** {@inheritDoc} */
	@Override
	public IFigure getContentPane() {
		return getLayer(ExtendedLayerRootEditPart.NODE_LAYER);
	}

	/** Adds all nodes and connectors as children. */
	@Override
	protected List<Object> getModelChildren() {
		List<Object> result = new ArrayList<Object>();
		result.addAll(modelElementHandler.getSubnodes(modelElement));
		result.addAll(modelElementHandler.getConnectors(modelElement));
		return result;
	}

	/** Sets anti-aliasing and the conenction router on connection layer. */
	private void setConnectionRouter() {
		ConnectionLayer cLayer = (ConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);
		if (cLayer != null) {
			cLayer.setAntialias(SWT.ON);
			cLayer.setConnectionRouter(connectionRouter);
		}
	}
}
