/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.fortiss.tooling.kernel.ui.extension.IModelElementHandler;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;

/**
 * The base class for all graphical edit parts. Any change to the underlying
 * model refreshes the edit part.
 * 
 * @param <T>
 *            the type of the model element wrapped.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 8835616CDDA5F811EE493732151D585F
 */
public abstract class GraphicalEditPartBase<T extends EObject> extends
		AbstractGraphicalEditPart {

	/**
	 * The adapter used to receive change notifications for the
	 * {@link #modelElement}.
	 */
	protected final Adapter fullRefreshAdapter = new AdapterImpl() {

		/** {@inheritDoc} */
		@Override
		public void notifyChanged(Notification notification) {
			// do not care what happens, just refresh everything
			refresh();
		}
	};

	/** The underlying model object. */
	protected final T modelElement;

	/** The model element handler to be used for the {@link #modelElement}. */
	protected final IModelElementHandler<T> modelElementHandler;

	/** Constructor. */
	@SuppressWarnings("unchecked")
	protected GraphicalEditPartBase(T modelElement) {
		this.modelElement = modelElement;
		setModel(modelElement);

		this.modelElementHandler = (IModelElementHandler<T>) IModelElementHandlerService.INSTANCE
				.getModelElementHandler(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	protected abstract void createEditPolicies();

	/** {@inheritDoc} */
	@Override
	public void activate() {
		super.activate();
		modelElement.eAdapters().add(fullRefreshAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public void deactivate() {
		modelElement.eAdapters().remove(fullRefreshAdapter);
		super.deactivate();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("rawtypes")
	@Override
	public Object getAdapter(Class key) {
		// EObjectSelectionUtils requires us to adapt to EObject, since
		// graphical edit parts can be provided as selection as well.
		if (key == EObject.class) {
			return modelElement;
		}
		return super.getAdapter(key);
	}

	/** Returns the model element of this edit part. */
	public T getModelElement() {
		return modelElement;
	}
}
