/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.layout.util;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.SnapToGrid;
import org.fortiss.tooling.base.ui.layout.constants.DefaultLayoutConstants;

/**
 * Utility methods for snap-to-grid support.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 864E908FB1B86BFCA206640785FAB8D0
 */
public final class SnapToGridUtils {

	/** Returns the given value aligned to the given grid size. */
	public static int snapToGrid(double d, int gridSize) {
		return gridSize * (int) Math.round(d / gridSize);
	}

	/** Returns the given value aligned to the recommended grid size. */
	public static int snapToGrid(double d) {
		return snapToGrid(d, DefaultLayoutConstants.DEFAULT_GRID_SIZE);
	}

	/**
	 * Modifies the x and y coordinates of the given point to be on the grid of
	 * the given size.
	 */
	public static void snapToGrid(Point p, int gridSize) {
		p.x = snapToGrid(p.x, gridSize);
		p.y = snapToGrid(p.y, gridSize);
	}

	/** Modifies the x and y coordinates of the given point to be on the grid. */
	public static void snapToGrid(Point p) {
		p.x = snapToGrid(p.x, DefaultLayoutConstants.DEFAULT_GRID_SIZE);
		p.y = snapToGrid(p.y, DefaultLayoutConstants.DEFAULT_GRID_SIZE);
	}

	/** Adapter code for snap to grid feature. */
	public static Object snapToGridAdapter(GraphicalEditPart editPart) {
		final Boolean val = (Boolean) editPart.getViewer().getProperty(
				SnapToGrid.PROPERTY_GRID_ENABLED);
		if (val != null && val.booleanValue()) {
			return new SnapToGrid(editPart);
		}
		return null;
	}
}
