/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.service;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.IElementCompositor;
import org.fortiss.tooling.kernel.extension.data.IElementCompositionContext;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.internal.ElementCompositorService;

/**
 * The element compositor service provides registration and access to model
 * element compositors. A compositor knows how to create and delete a link
 * between a parent element and its children. The formed relation usually
 * corresponds to an aggregation between the respective classes of the
 * meta-model.
 * 
 * @see IElementCompositor
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 72BAA99C391B2206BCB495943F28FBF5
 */
public interface IElementCompositorService {

	/** Returns the singleton instance of the service. */
	public static final IElementCompositorService INSTANCE = new ElementCompositorService();

	/**
	 * Determines if there is a compositor that allows the composition of the
	 * container and the prototype {@link EObject}.
	 */
	boolean canCompose(EObject container, EObject element,
			IElementCompositionContext context);

	/**
	 * Composes the container and the element. Since this operation may be
	 * canceled by the user, it returns a boolean value.
	 */
	boolean compose(EObject container, EObject element,
			IElementCompositionContext context);

	/**
	 * Returns whether given prototype can be composed with containers of the
	 * given class. This method searches for an {@link IElementCompositor} that
	 * is able to handle the composition in general.
	 * 
	 * @see IElementCompositor#canComposePrototype(Prototype)
	 */
	boolean canComposePrototype(Class<? extends EObject> containerClass,
			Prototype prototype);

	/**
	 * Determines if there is a compositor that allows the decomposition of the
	 * container and the contained {@link EObject}.
	 */
	boolean canDecompose(EObject container, EObject contained,
			IElementCompositionContext context);

	/**
	 * Decomposes the container and the element. Since this operation may be
	 * canceled by the user, it returns a boolean value.
	 */
	boolean decompose(EObject container, EObject contained,
			IElementCompositionContext context);
}
